pub enum DiagType {
NonUnit,
Unit,
}Expand description
Specifies whether a triangular matrix has an implicit unit diagonal.
Variants
NonUnit
The diagonal entries are stored explicitly and used as-is.
Unit
The diagonal is implicitly all ones (unit diagonal).